﻿#define DEBUGFORM
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace mPCIE_ADIO16_8F
{
    public enum OutputRange
    {
        SpanCode0,
        SpanCode1,
        SpanCode2,
        SpanCode3,
        SpanCode4
    }
    public enum PGIA
    {
        PGIA0,
        PGIA1,
        PGIA2,
        PGIA3,
        PGIA4,
        PGIA5,
        PGIA6,
        PGIA7
    }          //PGIA0 --> ±24.576V , PGIA1 -->> ±10.24V, PGIA2 --> ±5.12V, PGIA3 --> ±2.56V, PGIA4 --> ±1.28V, PGIA5 --> ±0.64V, PGIA6 --> Not used, PGIA7 --> ±20.48V
    public enum INX
    {
        IN0,
        IN1,
        IN2,
        IN3,
        IN4,
        IN5,
        IN6,
        IN7
    }       //IN0 --> 0, IN1 --> 1, IN2 --> 2, IN3 --> 3, IN4 --> 4, IN5 --> 5, IN6 --> 6, IN7 --> 7,
    static class RegisterOffset
    {
        public const UInt32 RESET_REG = 0x0;
        public const UInt32 BAR_REGISTER = 0x2;
        public const UInt32 WRITE_OFFSET = 0x4;
        public const UInt32 DEVICE_FREQUENCY_OFFSET = 0xC; // 50MHZ
        public const UInt32 ADC_RATE_DIVISOR_OFFSET = 0x10;
        public const UInt32 ADC_GAIN = 0x18;
        public const UInt32 FIFO_THRESHOLD_OFFSET = 0x20;
        public const UInt32 DATA_STATUS = 0x30;
        public const UInt32 ADC_CONTINUOUS_CONTROL = 0x38;
        public const UInt32 IRQ_CONTROL_REG = 0x40;   // <----  0x40 / 4 = 0x10 ~ 16(register)
        public const UInt32 DIO_DATA = 0x44;
        public const UInt32 DIO_CONFIGURE = 0x48;       
        
    }
    static class FileName
    {
        public static string ADCLOG_FILE = "ADC_LOGS.txt";
    }

    static class LSBVoltage
    {
        public const Double RangeTwelve = 750;
        public const Double RangeTen = 625;
        public const Double RangeFive = 312.5;
        public const Double RangeTwoPtFive = 156.25;
        public const Double RangeOnePtTwoFive = 78.125;
        public const Double RangePtSixOneTwoFive = 39.0625;
        public const Double RangePtThreeOneTwoFive = 19.53125;
    }

    public static class SpanCode
    {
        public const UInt32 WRITE_UPDATE_N = 0x3;
        public const UInt32 SPAN_TO_N = 0x6;
    }

    public static class ADCContinuousControl
    {
        public const int CPHA = 0X1;
        public const int Conversion_Mode_Select    = 0X1;
        public const int REFEN = 0X1;
        public const int Temperature_Sensor_Enable = 0X1;
        public const int Multiplexer = 0X1;
        public const int RSV = 0X1;
        public const int CFG = 0X1;
        public const int CONFIGURATION_LOAD_EXECUTE = 0X1;
        public const int CONFIGURATION_LOAD_STOP= 0X0;
    }
    public class ADC_GAIN_REGISTER
    {
        public const Int32 PGIA_GAIN_2 = 4;  
        public const Int32 PGIA_GAIN_1 = 8;
        public const Int32 PGIA_GAIN_7 = 16;
        public const Int32 PGIA_GAIN_6 = 20;
        public const Int32 PGIA_GAIN_5 = 24;
        public const Int32 PGIA_GAIN_4 = 28;
    }


    public class ADC_CONFIGURATION
    {
        public const Int32 CMS   = 1;
        public const Int32 TEMPB = 3;
        public const Int32 SEQ   = 4;
        public const Int32 MUX   = 6;
        public const Int32 PGIA_GAIN_0 = 7;
        public const Int32 COM         = 11;
        public const Int32 GAIN_CHANNEL= 12;
        public const Int32 START_STOP  = 16;
        public const Int32 CONFIG_LOAD = 17;


    }
    public class DAC_CONFIGURATION
    {
        public const Int32 DAC_INDEX_INDICATOR = 16;
        public const Int32 SPAN_UPDATE = 20;
    }

    public enum BAR { BAR_0, BAR_1, BAR_2, BAR_3, BAR_4, BAR_5, MAX_BAR_NB };

    public enum INTERRUPTTYPE { PIN_BASED_INT, MSI_INT, MSI_X_INT };

    public enum ISR { ISR_READ, ISR_WRITE, ISR_RW };

    public enum DMA
    {
        DMA_H2C_LSB_ADDR = 0x00 / 4,
        DMA_H2C_MSB_ADDR = 0x04 / 4,
        DMA_H2C_SIZE = 0x08 / 4,
        DMA_H2C_CTRL = 0x0C / 4,
        DMA_C2H_LSB_ADDR = 0x10 / 4,
        DMA_C2H_MSB_ADDR = 0x14 / 4,
        DMA_C2H_SIZE = 0x18 / 4,
        DMA_C2H_CTRL = 0x1C / 4,
        MB_WR_ADDR = 0x20 / 4,
        MAIL_BOX_REGISTER = 0x24 / 4,
        MB_WR_CTRL = 0x28 / 4,
        MB_RD_ADDR = 0x34 / 4,
        MB_RD_SIZE = 0x38 / 4,
        COUNTER_IT = 0x3C / 4,
        //SOFT_RESET = 0x40 / 4,
        INT_REG = 0x34 / 4,
        IRQ_CONTROL = 0x40 / 4,
        STATUS_REG = 0x48 / 4,
        DMA_RD_LOCAL_ADDR = 0x50 / 4,
        DMA_WR_LOCAL_ADDR = 0x54 / 4,
    };

    public enum DMA_CTRL
    {
        DMA_CTRL_DMA_ENA = 0x0001,  // Interrupt enable (activée à la fin du transfert DMA)
        DMA_CTRL_DEMO_ENA = 0x0002, // DMA demo mode enable
        DMA_CTRL_START = 0x0004, // DMA START
        DMA_CTRL_ABORT = 0x0008, // DMA ABORT
        DMA_CTRL_SC_ENA = 0x0010, // Scatter Gather Enable
        DMA_CTRL_CH_STATUS = 0x0F00 // DMA CHANNEL STATUS
    };// DMA_CTRL_REG_MASK;
}
