﻿using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.ComponentModel;
using AIONet;

namespace mPCIE_ADIO16_8F
{
    using TWinErrCode = UInt32;
    using TBoardIndex = UInt64;
    using TChannelIndex = UInt32;

    /// <summary>
    /// Form for loading the form
    /// </summary>
    public partial class Form1 : Form
    {
        public UInt32 iBoard;
        public ComboBox[] ADCRanges = new ComboBox[8];
        public CheckBox[] DIOStates = new CheckBox[16];
        public ComboBox[] DACRanges = new ComboBox[4];
        public Label[] DACLabels = new Label[4];
        public string[] Volts_FinStr = new string[] { "0.00", "0.00", "0.00", "0.00", "0.00", "0.00", "0.00", "0.00" };
        public TrackBar[] DACSliders = new TrackBar[4];
        public static Double FrequencyRate = 1000000.0;

        #region GUI Initialization 
        /// <summary>
        /// Function for Intializing all the components of the Form.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Form1_Load helps to display the form for the first time.
        /// </summary>
        /// <param name="Sender"> Contains a reference to the control/object that raised the event for loading the form.</param>
        /// <param name="E"> Parameter called e that contains the event data.</param>
        private void Form1_Load(object Sender, EventArgs E)
        {
            // Array of the LED style checkboxes:
            DIOStates[0] = checkBoxA0_1;
            DIOStates[1] = checkBoxA0_2;
            DIOStates[2] = checkBoxA0_3;
            DIOStates[3] = checkBoxA0_4;
            DIOStates[4] = checkBoxA0_5;
            DIOStates[5] = checkBoxA0_6;
            DIOStates[6] = checkBoxA0_7;
            DIOStates[7] = checkBoxA0_8;
            DIOStates[8] = checkBoxB0_9;
            DIOStates[9] = checkBoxB0_10;
            DIOStates[10] = checkBoxB0_11;
            DIOStates[11] = checkBoxB0_12;
            DIOStates[12] = checkBoxB0_13;
            DIOStates[13] = checkBoxB0_14;
            DIOStates[14] = checkBoxB0_15;
            DIOStates[15] = checkBoxB0_16;

            ADCRanges[0] = RangeComboBox0;
            ADCRanges[1] = RangeComboBox1;
            ADCRanges[2] = RangeComboBox2;
            ADCRanges[3] = RangeComboBox3;
            ADCRanges[4] = RangeComboBox4;
            ADCRanges[5] = RangeComboBox5;
            ADCRanges[6] = RangeComboBox6;
            ADCRanges[7] = RangeComboBox7;

            DACRanges[0] = DACRange0;
            DACRanges[1] = DACRange1;
            DACRanges[2] = DACRange2;
            DACRanges[3] = DACRange3;

            DACLabels[0] = DACLabel0;
            DACLabels[1] = DACLabel1;
            DACLabels[2] = DACLabel2;
            DACLabels[3] = DACLabel3;

            DACSliders[0] = DACCounts0;
            DACSliders[1] = DACCounts1;
            DACSliders[2] = DACCounts2;
            DACSliders[3] = DACCounts3;

            iBoard = 0; // Initialize default Device Only
            RangeComboBox0.SelectedIndex = 2;
            RangeComboBox1.SelectedIndex = 2;
            RangeComboBox2.SelectedIndex = 2;
            RangeComboBox3.SelectedIndex = 2;
            RangeComboBox4.SelectedIndex = 2;
            RangeComboBox5.SelectedIndex = 2;
            RangeComboBox6.SelectedIndex = 2;
            RangeComboBox7.SelectedIndex = 2;

            ADCLastChannel.SelectedIndex = 7;

            DACRange0.SelectedIndex = 3;
            DACRange1.SelectedIndex = 3;
            DACRange2.SelectedIndex = 3;
            DACRange3.SelectedIndex = 3;
        }

        /// <summary>
        /// Show_UIButton Method is used to enable and disable some of ADC buttons
        /// </summary>
        /// <param name="Status">Based on Status parameter ADC Buttons are enabled or disabled </param>
        public void Show_UIButton(bool Status)
        {
            if (Status == true)
            {
                ExecuteButton.Enabled = true;
                ADCLastChannel.Enabled = true;
                cbDifferential.Enabled = true;

                RangeComboBox0.Enabled = true;
                RangeComboBox1.Enabled = true;
                RangeComboBox2.Enabled = true;
                RangeComboBox3.Enabled = true;
                RangeComboBox4.Enabled = true;
                RangeComboBox5.Enabled = true;
                RangeComboBox6.Enabled = true;
                RangeComboBox7.Enabled = true;
            }
            else
            {
                ExecuteButton.Enabled = false;
                ADCLastChannel.Enabled = false;
                cbDifferential.Enabled = false;

                RangeComboBox0.Enabled = false;
                RangeComboBox1.Enabled = false;
                RangeComboBox2.Enabled = false;
                RangeComboBox3.Enabled = false;
                RangeComboBox4.Enabled = false;
                RangeComboBox5.Enabled = false;
                RangeComboBox6.Enabled = false;
                RangeComboBox7.Enabled = false;
            }
        }
        #endregion

        #region Analog Inputs
        /// <summary>
        /// ADCLastChannel_SelectedIndexChanged method is used when index of dropdown box is changed.
        /// </summary>
        /// <param name="Sender">Contains a reference to the control/object that raised the event 
        /// when index is changed in dropdown box.</param>
        /// <param name="E">Parameter called e that contains the event data</param>
        private void ADCLastChannel_SelectedIndexChanged(object Sender, EventArgs E)
        {
            AIO.ADC_SetChannelCount(iBoard, AIO.Sequencer.First, Convert.ToUInt32((Sender as ComboBox).SelectedIndex)+1);
        }

        /// <summary>
        /// ADCRangeComboBox_SelectedIndexChanged method is called when different ranges are selected
        /// </summary>
        /// <param name="Sender">Contains a reference to the control/object that raised the event 
        /// when there is a change in dropdown index box.</param>
        /// <param name="E">Parameter called e that contains the event data.</param>
        private void ADCRangeComboBox_SelectedIndexChanged(object Sender, EventArgs E)
        {
            AIO.Range Range = (AIO.Range)(Sender as ComboBox).SelectedIndex;
            TChannelIndex Channel = Convert.ToUInt32((Sender as ComboBox).Tag);
            AIO.ADC_SetRange1(iBoard, AIO.Sequencer.First, Channel, Range); // always use First Sequencer on mPCIe-ADIO!
        }

        private void Differential_CheckedChanged(object Sender, EventArgs E)
        {
            bool Differential = (Sender as CheckBox).Checked;

            int currentLastChannel = ADCLastChannel.SelectedIndex;

            ADCLastChannel.Items.Clear();
            ADCLastChannel.Items.Add("0"); 
            ADCLastChannel.Items.Add("1"); 
            ADCLastChannel.Items.Add("2"); 
            ADCLastChannel.Items.Add("3");
            if (Differential == false)
            {
                ADCLastChannel.Items.Add("4"); 
                ADCLastChannel.Items.Add("5"); 
                ADCLastChannel.Items.Add("6"); 
                ADCLastChannel.Items.Add("7");
            }
            ADCRanges[4].Visible = !Differential;
            ADCRanges[5].Visible = !Differential;
            ADCRanges[6].Visible = !Differential;
            ADCRanges[7].Visible = !Differential;
            labelCH4.Visible = !Differential;
            labelCH5.Visible = !Differential;
            labelCH6.Visible = !Differential;
            labelCH7.Visible = !Differential;

            ADCLastChannel.SelectedIndex = Math.Min(currentLastChannel, ADCLastChannel.Items.Count - 1);

            AIO.ADC_SetDifferential(iBoard, Differential);
        }

        private void ADCDisplayTimer_Tick(object sender, EventArgs e)
        {
            double[] Volts = new double[8];
            double[] Current = new double[8];
            AIO.ADC_GetImmediateScanV(iBoard, Volts, AIO.Range.NoChange);
            listBoxVoltage.Items.Clear();

            int HighChannel = ADCLastChannel.SelectedIndex;
            for (int channel = 0; channel < HighChannel+1; channel++)
            {
                Current[channel] = (Volts[channel] + 2.5) * 4.0;  // ±2.5V -> 0-20mA via 0-5V
                    
                listBoxVoltage.Items.Add("CH: " + channel.ToString() + " = " + Current[channel].ToString("F1") + " mA");
            }
        }

        private void ADCExecuteButton_Click(object sender, EventArgs e)
        {
            DisplayTimer.Enabled = true;

            ExecuteButton.Enabled = false;
            StopButton.Enabled = true;
        }

        private void ADCStopButton_Click(object sender, EventArgs e)
        {
            DisplayTimer.Enabled = false;

            StopButton.Enabled = false;
            ExecuteButton.Enabled = true;
        }
        #endregion

        #region Digital Input/Outputs
        private void PollDIOCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            DIOTimer.Enabled = PollDIOCheckBox.Checked;
        }

        private void DIOTimer_Tick(object sender, EventArgs e)
        {
            UInt32 bitLevels = AIO.PIO_In32(iBoard, RegisterOffset.DIO_DATA);
            for (int bitIndex = 0; bitIndex < 16; ++bitIndex)
            {
                int bitMask = 1 << (int)bitIndex;
                DIOStates[bitIndex].Checked = (bitLevels & bitMask) > 0;
            }
        }

        /// <summary>
        /// InOut_Direction_Click method is called when In or 
        /// Out button is pressed to change the direction of I/O Group 1-8 set.
        /// </summary>
        /// <param name="Sender">Contains a reference to the control/object that raised the event 
        /// when In or Out button is pressed</param>
        /// <param name="E"></param>
        private void InOut_Direction_Click(object Sender, EventArgs E)
        {
            Button InOut_DirectionButton = Sender as Button;
            int IOGroupIndex = Convert.ToInt32(InOut_DirectionButton.Tag);
            bool Output = InOut_DirectionButton.Text == "Set Out";
            InOut_DirectionButton.Text = Output ? "Set In" : "Set Out";
            //AIO.DIO_Configure1(iBoard, IOGroupIndex, Output);// Not Yet Implemented
            UInt32 DIOControlRegister = AIO.PIO_In32(iBoard, RegisterOffset.DIO_CONFIGURE);
            int bitMask = 1 << IOGroupIndex;
            if (IOGroupIndex > 0) bitMask <<= 7; // IOGroups 1-8 are controlled by bits 8-15
            if (Output) 
                DIOControlRegister |= (uint)bitMask; 
            else 
                DIOControlRegister &= ~(uint)bitMask;
            AIO.PIO_Out32(iBoard, RegisterOffset.DIO_CONFIGURE, DIOControlRegister);
        }

        /// <summary>
        /// DIO_Checkbox_CheckedChanged method is used to check which DIO checkbox data is changed and 
        /// then setting boolean value according to the user selection 
        /// </summary>
        /// <param name="Sender">Contains a reference to the control/object that raised the 
        /// event when DIO checkbox value has changed</param>
        /// <param name="E">Parameter called e that contains the event data.</param>
        private void DIO_Checkbox_CheckedChanged(object Sender, EventArgs E)
        {
            CheckBox DIO_Checkbox = Sender as CheckBox;
            TChannelIndex bitIndex = Convert.ToUInt32(DIO_Checkbox.Tag);
            bool bSet = DIO_Checkbox.Checked;
            // AIO.DIO_Output1(iBoard, bitIndex, high); // Not Yet Implemented
            int bitMask = 1 << (int)bitIndex;
            UInt32 bitLevels = AIO.PIO_In32(iBoard, RegisterOffset.DIO_DATA);
            if (bSet) 
            { bitLevels |= (uint)bitMask; } 
            else 
            { bitLevels &= ~(uint)bitMask; }
            AIO.PIO_Out32(iBoard, RegisterOffset.DIO_DATA, bitLevels);
        }

        private void PollDIOCheckBox_CheckedChanged_1(object sender, EventArgs e)
        {
            DIOGroupBox.Visible = PollDIOCheckBox.Checked;
            DIOTimer.Enabled = PollDIOCheckBox.Checked;
        }
        #endregion

        #region Analog Outputs
        private double DACCountsToVolts(UInt16 Counts, AIO.Range Range)
        {
            double Volts = 1.0 / 0x10000 * Counts; // Volts is now 0..1
            switch(Range) // all bipolar ranges are double span compared to unipolar ranges
            {
                case AIO.Range.Bipolar10:
                case AIO.Range.Bipolar2_5:
                case AIO.Range.Bipolar5:
                case AIO.Range.Code2:
                case AIO.Range.Code3:
                case AIO.Range.Code4:
                    Volts = Volts * 2.0 - 1;
                    break;
                default:
                    break;
            };
            switch(Range) // multiply by the max voltage per range
            {
                case AIO.Range.Code0:
                case AIO.Range.Code2:
                case AIO.Range.Unipolar5:
                case AIO.Range.Bipolar5:
                    Volts *= 5.0;
                    break;
                case AIO.Range.Code1:
                case AIO.Range.Code3:
                case AIO.Range.Bipolar10:
                case AIO.Range.Unipolar10:
                    Volts *= 10.0;
                    break;
                case AIO.Range.Bipolar2_5:
                case AIO.Range.Code4:
                    Volts *= 2.5;
                    break;
            };
            return Volts;
        }

        /// <summary>
        /// DACValueChanged method is called when value is changed on the trackbar
        /// </summary>
        /// <param name="Sender">Contains a reference to the control/object that raised the event when trackbar is moved</param>
        /// <param name="E">Parameter called e that contains the event data</param>
        private void DACValueChanged(object Sender, EventArgs E)
        {
            TrackBar bar = Sender as TrackBar;
            TChannelIndex Channel = Convert.ToUInt32(bar.Tag);
            UInt16 Counts = Convert.ToUInt16(bar.Value);
            //AIO.DAC_OutputCounts(iBoard, Channel, Counts); // optionally, use Counts
            AIO.Range Range = (AIO.Range)DACRanges[Channel].SelectedIndex;
            double Volts = DACCountsToVolts(Counts, Range);
            UInt32 status = AIO.DAC_OutputV(iBoard, Channel, Volts);
            DACLabels[Channel].Text = Volts.ToString();
        }

        /// <summary>
        /// DACRange_SelectedIndexChanged method is called, when position of trackbar cursor (or) 
        /// Existing range of channel  will change.
        /// </summary>
        /// <param name="Sender">Contains a reference to the control/object that raised the event</param>
        /// <param name="E">Parameter called e that contains the event data</param>
        private void DACRange_SelectedIndexChanged(object Sender, EventArgs E)
        {
            TChannelIndex Ch = Convert.ToUInt32((Sender as ComboBox).Tag);
            AIO.Range RangeCode = (AIO.Range)(Sender as ComboBox).SelectedIndex;
            AIO.DAC_SetRange1(iBoard, Ch, RangeCode);
            DACValueChanged(DACSliders[Ch], E);
        }
        #endregion

    }
}
